"""
Contains Marktteilnehmer class
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..enum.marktrolle import Marktrolle
    from ..enum.rollencodetyp import Rollencodetyp
    from ..enum.sparte import Sparte
    from .geschaeftspartner import Geschaeftspartner


@postprocess_docstring
class Marktteilnehmer(Geschaeftsobjekt):
    """
    Objekt zur Aufnahme der Information zu einem Marktteilnehmer

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Marktteilnehmer.svg" type="image/svg+xml"></object>

    .. HINT::
        `Marktteilnehmer JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Marktteilnehmer.json>`_

    """

    typ: Annotated[Literal[BoTyp.MARKTTEILNEHMER], Field(alias="_typ")] = BoTyp.MARKTTEILNEHMER
    marktrolle: Optional["Marktrolle"] = None
    """Gibt im Klartext die Bezeichnung der Marktrolle an"""
    rollencodenummer: Optional[str] = None
    """Gibt die Codenummer der Marktrolle an"""
    rollencodetyp: Optional["Rollencodetyp"] = None
    """Gibt den Typ des Codes an"""
    sparte: Optional["Sparte"] = None
    """Sparte des Marktteilnehmers, z.B. Gas oder Strom"""
    makoadresse: Optional[list[str]] = None
    """Die 1:1-Kommunikationsadresse des Marktteilnehmers. Diese wird in der Marktkommunikation verwendet. Konkret kann dies eine eMail-Adresse oder ein AS4-Endpunkt sein."""
    geschaeftspartner: Optional["Geschaeftspartner"] = None
    """Der zu diesem Marktteilnehmer gehörende Geschäftspartner"""



import pytest

from bo4e import Konfigurationsprodukt, Marktteilnehmer
from tests.serialization_helper import assert_serialization_roundtrip


class TestKonfigurationsprodukt:
    @pytest.mark.parametrize(
        "konfigurationsprodukt",
        [
            pytest.param(
                Konfigurationsprodukt(
                    produktcode="1212121212",
                    leistungskurvendefinition="",
                    schaltzeitdefinition="",
                    marktpartner=Marktteilnehmer(),
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, konfigurationsprodukt: Konfigurationsprodukt) -> None:
        """
        Test de-/serialisation of Konfigurationsprodukt.
        """

        assert_serialization_roundtrip(konfigurationsprodukt)



from datetime import datetime, timezone

import pytest

from bo4e import (
    Anrede,
    Geschaeftspartner,
    Geschaeftspartnerrolle,
    Kontaktweg,
    Organisationstyp,
    Person,
    Sparte,
    Titel,
    Unterschrift,
    Vertrag,
    Vertragsart,
    Vertragskonditionen,
    Vertragsstatus,
    Vertragsteil,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestVertrag:
    @pytest.mark.parametrize(
        "vertrag",
        [
            pytest.param(
                Vertrag(
                    vertragsnummer="1234567891011",
                    vertragsart=Vertragsart.BILANZIERUNGSVERTRAG,
                    vertragsstatus=Vertragsstatus.AKTIV,
                    sparte=Sparte.STROM,
                    vertragsbeginn=datetime(2021, 4, 30, 13, 45, tzinfo=timezone.utc),
                    vertragsende=datetime(2021, 6, 5, 16, 30, tzinfo=timezone.utc),
                    vertragspartner1=Geschaeftspartner(
                        ansprechpartner=[Person(), Person()],
                        anrede=Anrede.EHELEUTE,
                        individuelle_anrede="Künstler",
                        titel=Titel.PROF_DR,
                        vorname="Hans",
                        nachname="Müller-Schmidt",
                        organisationstyp=Organisationstyp.PRIVATPERSON,
                        kontaktwege=[Kontaktweg()],
                        umsatzsteuer_id="DE267311963",
                        glaeubiger_id="DE98ZZZ09999999999",
                        website="bo4e.de",
                        geschaeftspartnerrollen=[Geschaeftspartnerrolle.DIENSTLEISTER],
                    ),
                    vertragspartner2=Geschaeftspartner(
                        ansprechpartner=[Person(), Person()],
                        organisationstyp=Organisationstyp.UNTERNEHMEN,
                        organisationsname="Hochfrequenz",
                        geschaeftspartnerrollen=[Geschaeftspartnerrolle.DIENSTLEISTER],
                    ),
                    vertragsteile=[
                        Vertragsteil(
                            vertragsteilbeginn=datetime(2021, 4, 30, tzinfo=timezone.utc),
                            vertragsteilende=datetime(2021, 6, 5, tzinfo=timezone.utc),
                        ),
                        Vertragsteil(
                            vertragsteilbeginn=datetime(2001, 1, 23, tzinfo=timezone.utc),
                            vertragsteilende=datetime(2002, 12, 3, tzinfo=timezone.utc),
                        ),
                    ],
                    beschreibung="Hello Vertrag",
                    vertragskonditionen=Vertragskonditionen(beschreibung="Beschreibung"),
                    unterzeichnervp1=[Unterschrift(name="Foo")],
                    unterzeichnervp2=[Unterschrift(name="Bar"), Unterschrift(name="Dr.No")],
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, vertrag: Vertrag) -> None:
        """
        Test de-/serialisation of Vertrag.
        """
        assert_serialization_roundtrip(vertrag)



"""
Contains Vertragsteil class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:

    from .menge import Menge

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Vertragsteil(COM):
    """
    Abbildung für einen Vertragsteil. Der Vertragsteil wird dazu verwendet,
    eine vertragliche Leistung in Bezug zu einer Lokation (Markt- oder Messlokation) festzulegen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Vertragsteil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertragsteil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Vertragsteil.json>`_

    """

    typ: Annotated[Literal[ComTyp.VERTRAGSTEIL], Field(alias="_typ")] = ComTyp.VERTRAGSTEIL

    vertragsteilbeginn: Optional[pydantic.AwareDatetime] = None
    """
    Start der Gültigkeit des Vertragsteils (inklusiv)
    """
    vertragsteilende: Optional[pydantic.AwareDatetime] = None
    """
    Ende der Gültigkeit des Vertragsteils (exklusiv)
    """

    lokation: Optional[str] = None
    """
    Der Identifier für diejenigen Markt- oder Messlokation, die zu diesem Vertragsteil gehören.
    Verträge für mehrere Lokationen werden mit mehreren Vertragsteilen abgebildet
    """
    vertraglich_fixierte_menge: Optional["Menge"] = None
    """
    Für die Lokation festgeschriebene Abnahmemenge
    """
    minimale_abnahmemenge: Optional["Menge"] = None
    """
    Für die Lokation festgelegte Mindestabnahmemenge (inklusiv)
    """
    maximale_abnahmemenge: Optional["Menge"] = None
    """
    Für die Lokation festgelegte maximale Abnahmemenge (exklusiv)
    """



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Messart(StrEnum):
    """
    Gibt an, auf welche Art gemessen wurde.
    """

    AKTUELLERWERT = "AKTUELLERWERT"
    """AKTUELLERWERT"""
    MITTELWERT = "MITTELWERT"
    """MITTELWERT"""
    MAXIMALWERT = "MAXIMALWERT"
    """MAXIMALWERT"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Vertragsart(StrEnum):
    """
    Aufzählung der Vertragsarten.
    """

    ENERGIELIEFERVERTRAG = "ENERGIELIEFERVERTRAG"
    """Energieliefervertrag"""
    NETZNUTZUNGSVERTRAG = "NETZNUTZUNGSVERTRAG"
    """Netznutzungsvertrag"""
    BILANZIERUNGSVERTRAG = "BILANZIERUNGSVERTRAG"
    """Bilanzierungsvertrag"""
    MESSSTELLENBETRIEBSVERTRAG = "MESSSTELLENBETRIEBSVERTRAG"
    """Messstellenabetriebsvertrag"""
    BUENDELVERTRAG = "BUENDELVERTRAG"
    """Bündelvertrag"""



import pytest

from bo4e import (
    Konfigurationsprodukt,
    Lokationszuordnung,
    Marktrolle,
    Menge,
    Netzlokation,
    Sparte,
    Verwendungszweck,
    VerwendungszweckProMarktrolle,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestNetzlokation:
    @pytest.mark.parametrize(
        "netzlokation",
        [
            pytest.param(
                Netzlokation(
                    netzlokations_id="3784658734657",
                    sparte=Sparte.GAS,
                    netzanschlussleistung=Menge(),
                    grundzustaendiger_msb_codenr="1829371872392",
                    steuerkanal=False,
                    obiskennzahl="82376487236",
                    verwendungszweck=VerwendungszweckProMarktrolle(
                        marktrolle=Marktrolle.LF,
                        Zwecke=[Verwendungszweck.BILANZKREISABRECHNUNG],
                    ),
                    konfigurationsprodukte=[Konfigurationsprodukt()],
                    eigenschaft_msb_lokation=Marktrolle.LF,
                    lokationsbuendel_objektcode="9992 00000 125 6",
                    lokationszuordnungen=[Lokationszuordnung()],
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, netzlokation: Netzlokation) -> None:
        """
        Test de-/serialisation of Netzlokation.
        """
        assert_serialization_roundtrip(netzlokation)



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Gasqualitaet(StrEnum):
    """
    Unterscheidung für hoch- und niedrig-kalorisches Gas.
    """

    H_GAS = "H_GAS"
    """High Caloric Gas"""
    L_GAS = "L_GAS"
    """Low Caloric Gas"""



"""
Contains Tarifpreisblatt class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .tarifinfo import Tarifinfo

if TYPE_CHECKING:
    from ..com.aufabschlag import AufAbschlag
    from ..com.preisgarantie import Preisgarantie
    from ..com.tarifberechnungsparameter import Tarifberechnungsparameter
    from ..com.tarifeinschraenkung import Tarifeinschraenkung
    from ..com.tarifpreisposition import Tarifpreisposition


# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Tarifpreisblatt(Tarifinfo):
    """
    Tarifinformation mit Preisen, Aufschlägen und Berechnungssystematik

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Tarifpreisblatt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreisblatt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Tarifpreisblatt.json>`_

    """

    typ: Annotated[Literal[BoTyp.TARIFPREISBLATT], Field(alias="_typ")] = BoTyp.TARIFPREISBLATT  # type: ignore[assignment]
    # required attributes (additional to those of Tarifinfo)
    preisstand: Optional[pydantic.AwareDatetime] = None
    """Gibt an, wann der Preis zuletzt angepasst wurde"""
    tarifpreise: Optional[list["Tarifpreisposition"]] = None
    """Die festgelegten Preise, z.B. für Arbeitspreis, Grundpreis etc."""
    berechnungsparameter: Optional["Tarifberechnungsparameter"] = None
    """Für die Berechnung der Kosten sind die hier abgebildeten Parameter heranzuziehen"""

    tarifeinschraenkung: Optional["Tarifeinschraenkung"] = None
    """Die Bedingungen und Einschränkungen unter denen ein Tarif angewendet werden kann"""
    preisgarantie: Optional["Preisgarantie"] = None
    """Festlegung von Garantien für bestimmte Preisanteile"""
    tarif_auf_abschlaege: Optional[list["AufAbschlag"]] = None
    """Auf- und Abschläge auf die Preise oder Kosten"""



"""
Contains Betrag class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.waehrungscode import Waehrungscode


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Betrag(COM):
    """
    Die Komponente wird dazu verwendet, Summenbeträge (beispielsweise in Angeboten und Rechnungen) als Geldbeträge
    abzubilden. Die Einheit ist dabei immer die Hauptwährung also Euro, Dollar etc…

    .. raw:: html

        <object data="../_static/images/bo4e/com/Betrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Betrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Betrag.json>`_

    """

    typ: Annotated[Literal[ComTyp.BETRAG], Field(alias="_typ")] = ComTyp.BETRAG

    wert: Optional[Decimal] = None
    """Gibt den Betrag des Preises an."""
    waehrung: Optional["Waehrungscode"] = None
    """Die entsprechende Waehrung"""



"""
Contains Verwendungszweck class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.marktrolle import Marktrolle
    from ..enum.verwendungszweck import Verwendungszweck


# pylint: disable=too-few-public-methods


@postprocess_docstring
class VerwendungszweckProMarktrolle(COM):
    """
    Dient zur Identifizierung des Verwendungszwecks der Marktrolle an der Marktlokation, der die Werte zu übermitteln sind.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Verwendungszweck.svg" type="image/svg+xml"></object>

    .. HINT::
        `Verwendungszweck JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Verwendungszweck.json>`_

    """

    typ: Annotated[Literal[ComTyp.VERWENDUNGSZWECKPROMARKTROLLE], Field(alias="_typ")] = (
        ComTyp.VERWENDUNGSZWECKPROMARKTROLLE
    )

    marktrolle: Optional["Marktrolle"] = None
    """
    Marktrolle, für die die Daten relevant sind
    """
    Zwecke: Optional[list["Verwendungszweck"]] = None
    """
    Verwendungszwecke
    """



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Registeranzahl(StrEnum):
    """
    Die Registeranzahl wird verwendet zur Charakterisierung von Zählern und daraus resultierenden Tarifen.
    """

    EINTARIF = "EINTARIF"
    """Eintarif"""
    ZWEITARIF = "ZWEITARIF"
    """Zweitarif"""
    MEHRTARIF = "MEHRTARIF"
    """Mehrtarif"""



"""
Contains Zeitreihe class
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.zeitreihenwert import Zeitreihenwert
    from ..enum.medium import Medium
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.messart import Messart
    from ..enum.messgroesse import Messgroesse
    from ..enum.messwertstatus import Messwertstatus


@postprocess_docstring
class Zeitreihe(Geschaeftsobjekt):
    """
    Abbildung einer allgemeinen Zeitreihe mit einem Wertvektor.
    Die Werte können mit wahlfreier zeitlicher Distanz im Vektor abgelegt sein.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zeitreihe.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitreihe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Zeitreihe.json>`_

    """

    typ: Annotated[Literal[BoTyp.ZEITREIHE], Field(alias="_typ")] = BoTyp.ZEITREIHE
    bezeichnung: Optional[str] = None
    """Bezeichnung für die Zeitreihe"""
    messgroesse: Optional["Messgroesse"] = None
    """Beschreibt, was gemessen wurde (z.B. Strom, Spannung, Wirkleistung, Scheinleistung)"""
    messart: Optional["Messart"] = None
    """Beschreibt die Art der Messung (z.B. aktueller Wert, mittlerer Wert, maximaler Wert)"""
    medium: Optional["Medium"] = None
    """Medium, das gemessen wurde (z.B. Wasser, Dampf, Strom, Gas)"""
    einheit: Optional["Mengeneinheit"] = None
    """Alle Werte in der Tabelle haben die Einheit, die hier angegeben ist"""
    werte: Optional[list["Zeitreihenwert"]] = None
    """Hier liegen jeweils die Werte"""

    beschreibung: Optional[str] = None
    """Beschreibt die Verwendung der Zeitreihe"""
    version: Optional[str] = None
    """Version der Zeitreihe"""
    wertherkunft: Optional["Messwertstatus"] = None
    """Kennzeichnung, wie die Werte entstanden sind, z.B. ABGELESEN oder PROGNOSEWERT"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Zaehlergroesse(StrEnum):
    """
    Auflistung möglicher Größen von Zählern
    """

    G2KOMMAP5 = "G2KOMMA5"
    """Gaszähler Größe G2.5"""
    G4 = "G4"
    """Gaszähler Größe G4"""
    G6 = "G6"
    """Gaszähler Größe G6"""
    G10 = "G10"
    """Gaszähler Größe G10"""
    G16 = "G16"
    """Gaszähler Größe G16"""
    G25 = "G25"
    """Gaszähler Größe G25"""
    G40 = "G40"
    """Gaszähler Größe G40"""
    G65 = "G65"
    """Gaszähler Größe G65"""
    G100 = "G100"
    """Gaszähler Größe G100"""
    G160 = "G160"
    """Gaszähler Größe G160"""
    G250 = "G250"
    """Gaszähler Größe G250"""
    G400 = "G400"
    """Gaszähler Größe G400"""
    G650 = "G650"
    """Gaszähler Größe G650"""
    G1000 = "G1000"
    """Gaszähler Größe G1000"""
    G1600 = "G1600"
    """Gaszähler Größe G1600"""
    G2500 = "G2500"
    """Gaszähler Größe G2500"""
    G4000 = "G4000"
    """Gaszähler Größe G4000"""
    G6500 = "G6500"
    """Gaszähler Größe G6500"""
    G10000 = "G10000"
    """Gaszähler Größe G10000"""
    G12500 = "G12500"
    """Gaszähler Größe G12500"""
    G16000 = "G16000"
    """Gaszähler Größe G16000"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Kostenklasse(StrEnum):
    """
    Kostenklassen bilden die oberste Ebene der verschiedenen Kosten.
    In der Regel werden die Gesamtkosten einer Kostenklasse in einer App berechnet.
    """

    FREMDKOSTEN = "FREMDKOSTEN"
    """Fremdkosten"""
    BESCHAFFUNG = "BESCHAFFUNG"
    """Beschaffung"""
    SELBSTKOSTEN = "SELBSTKOSTEN"
    """Selbstkosten"""
    MARGEN = "MARGEN"
    """Margen"""
    ENERGIEVERSORGUNGSKOSTEN = "ENERGIEVERSORGUNGSKOSTEN"
    """Energieversorgungskosten"""



"""
Contains Tarifpreis class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .preis import Preis

if TYPE_CHECKING:
    from ..enum.preistyp import Preistyp


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Tarifpreis(Preis):
    """
    Abbildung eines Tarifpreises mit Preistyp und Beschreibung abgeleitet von COM Preis.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifpreis.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreis JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Tarifpreis.json>`_

    """

    typ: Annotated[Literal[ComTyp.TARIFPREIS], Field(alias="_typ")] = ComTyp.TARIFPREIS  # type:ignore[assignment]

    preistyp: Optional["Preistyp"] = None
    """ Angabe des Preistypes (z.B. Grundpreis)"""

    beschreibung: Optional[str] = None
    """ Beschreibung des Preises. Hier können z.B. Preisdetails angegeben sein, beispielsweise "Drehstromzähler"."""



"""
Contains Konfigurationsprodukt class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..bo.marktteilnehmer import Marktteilnehmer


# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Konfigurationsprodukt(COM):
    """
    Object containing information about a Konfigurationsprodukt

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Konfigurationsprodukt.svg" type="image/svg+xml"></object>

    .. HINT::
        `Konfigurationsprodukt JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Konfigurationsprodukt.json>`_

    """

    typ: Annotated[Literal[ComTyp.KONFIGURATIONSPRODUKT], Field(alias="_typ")] = ComTyp.KONFIGURATIONSPRODUKT

    produktcode: Optional[str] = None
    leistungskurvendefinition: Optional[str] = None
    schaltzeitdefinition: Optional[str] = None
    marktpartner: Optional["Marktteilnehmer"] = None



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class AufAbschlagstyp(StrEnum):
    """
    Festlegung, ob der Auf- oder Abschlag mit relativen oder absoluten Werten erfolgt.
    """

    RELATIV = "RELATIV"
    """prozentualer Auf-/Abschlag"""
    ABSOLUT = "ABSOLUT"
    """Absoluter Auf-/Abschlag"""



import pytest

from bo4e import (
    AufAbschlagProOrt,
    AufAbschlagRegional,
    AufAbschlagstyp,
    AufAbschlagsziel,
    Energiemix,
    Preisgarantie,
    Tarifeinschraenkung,
    Vertragskonditionen,
    Waehrungseinheit,
    Zeitraum,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestAufAbschlagRegional:
    @pytest.mark.parametrize(
        "aufabschlagregional",
        [
            pytest.param(
                AufAbschlagRegional(
                    bezeichnung="foo",
                    betraege=[AufAbschlagProOrt()],
                    beschreibung="bar",
                    auf_abschlagstyp=AufAbschlagstyp.RELATIV,
                    auf_abschlagsziel=AufAbschlagsziel.ARBEITSPREIS_HT,
                    einheit=Waehrungseinheit.EUR,
                    website="foo.bar",
                    zusatzprodukte=["Asterix", "Obelix"],
                    voraussetzungen=["Petterson", "Findus"],
                    tarifnamensaenderungen="foobar",
                    gueltigkeitszeitraum=Zeitraum(),
                    energiemixaenderung=Energiemix(),
                    vertagskonditionsaenderung=Vertragskonditionen(),
                    garantieaenderung=Preisgarantie(),
                    einschraenkungsaenderung=Tarifeinschraenkung(),
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, aufabschlagregional: AufAbschlagRegional) -> None:
        """
        Test de-/serialisation of AufAbschlagRegional with minimal attributes.
        """
        assert_serialization_roundtrip(aufabschlagregional)



"""
Contains Fremdkostenblock class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:

    from .betrag import Betrag
    from .fremdkostenposition import Fremdkostenposition

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Fremdkostenblock(COM):
    """
    Komponente zur Abbildung eines Kostenblocks in den Fremdkosten

    .. raw:: html

        <object data="../_static/images/bo4e/com/Fremdkostenblock.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkostenblock JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Fremdkostenblock.json>`_

    """

    typ: Annotated[Literal[ComTyp.FREMDKOSTENBLOCK], Field(alias="_typ")] = ComTyp.FREMDKOSTENBLOCK

    kostenblockbezeichnung: Optional[str] = None
    """Bezeichnung für einen Kostenblock. Z.B. Netzkosten, Messkosten, Umlagen, etc."""

    kostenpositionen: Optional[list["Fremdkostenposition"]] = None
    """
    Hier sind die Details zu einer Kostenposition aufgeführt. Z.B.:
    Alliander Netz Heinsberg GmbH, 2018-02-01, 2019-01-01, Arbeitspreis HT, 3.660 kWh,
    5,8200 ct/kWh, 213,01 €
    """

    summe_kostenblock: Optional["Betrag"] = None
    """Die Summe aller Kostenpositionen dieses Blocks"""
    # todo: write validator fo this sum, see https://github.com/Hochfrequenz/BO4E-python/issues/324



# pylint: disable=missing-module-docstring,line-too-long
from bo4e.enum.strenum import StrEnum


class Tarifkalkulationsmethode(StrEnum):
    """
    Auflistung der verschiedenen Berechnungsmethoden für ein Preisblatt.
    """

    KEINE = "KEINE"
    """Es wird keine Berechnung durchgeführt, sondern lediglich die Menge mit dem Preis multipliziert."""
    STAFFELN = "STAFFELN"
    """Staffelmodell, d.h. die Gesamtmenge wird in eine Staffel eingeordnet und für die gesamte Menge gilt der so ermittelte Preis"""
    ZONEN = "ZONEN"
    """Zonenmodell, d.h. die Gesamtmenge wird auf die Zonen aufgeteilt und für die Teilmengen gilt der jeweilige Preis der Zone."""
    BESTABRECHNUNG_STAFFEL = "BESTABRECHNUNG_STAFFEL"
    """Bestabrechnung innerhalb der Staffelung"""
    PAKETPREIS = "PAKETPREIS"
    """Preis für ein Paket (eine Menge)."""



"""
Contains StandorteigenschaftenGas class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:

    from .marktgebietinfo import MarktgebietInfo

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class StandorteigenschaftenGas(COM):
    """
    Standorteigenschaften der Sparte Gas

    .. raw:: html

        <object data="../_static/images/bo4e/com/StandorteigenschaftenGas.svg" type="image/svg+xml"></object>

    .. HINT::
        `StandorteigenschaftenGas JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/StandorteigenschaftenGas.json>`_

    """

    typ: Annotated[Literal[ComTyp.STANDORTEIGENSCHAFTENGAS], Field(alias="_typ")] = ComTyp.STANDORTEIGENSCHAFTENGAS
    netzkontonummern: Optional[list[str]] = None
    """Netzkontonummern der Gasnetze"""
    marktgebiete: Optional[list["MarktgebietInfo"]] = None
    """Die Informationen zu Marktgebieten in dem Netz."""



"""
Contains Energiemix class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.oekolabel import Oekolabel
    from ..enum.oekozertifikat import Oekozertifikat
    from ..enum.sparte import Sparte
    from .energieherkunft import Energieherkunft

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module


@postprocess_docstring
class Energiemix(COM):
    """
    Zusammensetzung der gelieferten Energie aus den verschiedenen Primärenergieformen.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Energiemix.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energiemix JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Energiemix.json>`_

    """

    typ: Annotated[Literal[ComTyp.ENERGIEMIX], Field(alias="_typ")] = ComTyp.ENERGIEMIX

    energiemixnummer: Optional[int] = None
    """Eindeutige Nummer zur Identifizierung des Energiemixes"""
    energieart: Optional["Sparte"] = None
    """Strom oder Gas etc."""
    bezeichnung: Optional[str] = None
    """Bezeichnung des Energiemix"""
    gueltigkeitsjahr: Optional[int] = None
    """Jahr, für das der Energiemix gilt"""
    anteil: Optional[list["Energieherkunft"]] = None
    """Anteile der jeweiligen Erzeugungsart"""

    bemerkung: Optional[str] = None
    """Bemerkung zum Energiemix"""
    co2_emission: Optional[Decimal] = None
    """Höhe des erzeugten CO2-Ausstosses in g/kWh"""
    atommuell: Optional[Decimal] = None
    """Höhe des erzeugten Atommülls in g/kWh"""
    oekozertifikate: Optional[list["Oekozertifikat"]] = None
    """Zertifikate für den Energiemix"""
    oekolabel: Optional[list["Oekolabel"]] = None
    """Ökolabel für den Energiemix"""
    ist_in_oeko_top_ten: Optional[bool] = None
    """Kennzeichen, ob der Versorger zu den Öko Top Ten gehört"""
    website: Optional[str] = None
    """Internetseite, auf der die Strommixdaten veröffentlicht sind"""



"""
Contains Zustaendigkeit class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.themengebiet import Themengebiet


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Zustaendigkeit(COM):
    """
    Enthält die zeitliche Zuordnung eines Ansprechpartners zu Abteilungen und Zuständigkeiten.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zustaendigkeit.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zustaendigkeit JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zustaendigkeit.json>`_

    """

    typ: Annotated[Literal[ComTyp.ZUSTAENDIGKEIT], Field(alias="_typ")] = ComTyp.ZUSTAENDIGKEIT
    themengebiet: Optional["Themengebiet"] = None
    """
    Hier kann eine thematische Zuordnung des Ansprechpartners bzw. der Person angegeben werden
    """

    position: Optional[str] = None
    """Berufliche Rolle des Ansprechpartners/ der Person"""
    abteilung: Optional[str] = None
    """Abteilung, in der der Ansprechpartner/ die Person tätig ist"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


# pylint: disable=line-too-long
class Geraetetyp(StrEnum):
    """
    Auflistung möglicher abzurechnender Gerätetypen.
    """

    MULTIPLEXANLAGE = "MULTIPLEXANLAGE"
    """Multiplexeranlage"""
    PAUSCHALANLAGE = "PAUSCHALANLAGE"
    """Pauschalanlagen"""
    VERSTAERKERANLAGE = "VERSTAERKERANLAGE"
    """Verstärkeranlage"""
    SUMMATIONSGERAET = "SUMMATIONSGERAET"
    """Summationsgerät"""
    IMPULSGEBER = "IMPULSGEBER"
    """Impulsgeber"""
    MENGENUMWERTER = "MENGENUMWERTER"
    """Mengenumwerter"""
    STROMWANDLER = "STROMWANDLER"
    """Stromwandler"""
    SPANNUNGSWANDLER = "SPANNUNGSWANDLER"
    """Spannungs-Wandler"""
    KOMBIMESSWANDLER = "KOMBIMESSWANDLER"
    """Kombimesswandler"""
    BLOCKSTROMWANDLER = "BLOCKSTROMWANDLER"
    """Blockstromwandler"""
    DATENLOGGER = "DATENLOGGER"
    """Datenlogger"""
    KOMMUNIKATIONSANSCHLUSS = "KOMMUNIKATIONSANSCHLUSS"
    """Kommunikationsanschluss"""
    MODEM = "MODEM"
    """Modem"""
    TELEKOMMUNIKATIONSEINRICHTUNG = "TELEKOMMUNIKATIONSEINRICHTUNG"
    """vom Messstellenbetreiber beigestellte Telekommunikationseinrichtung (Telefonanschluss)"""
    MODERNE_MESSEINRICHTUNG = "MODERNE_MESSEINRICHTUNG"
    """moderne Messeinrichtung"""
    INTELLIGENTES_MESSYSTEM = "INTELLIGENTES_MESSYSTEM"
    """Intelligentes Messystem"""
    STEUEREINRICHTUNG = "STEUEREINRICHTUNG"
    """Steuereinrichtung"""
    TARIFSCHALTGERAET = "TARIFSCHALTGERAET"
    """Tarifschaltgerät"""
    RUNDSTEUEREMPFAENGER = "RUNDSTEUEREMPFAENGER"
    """Rundsteuerempfänger"""
    OPTIONALE_ZUS_ZAEHLEINRICHTUNG = "OPTIONALE_ZUS_ZAEHLEINRICHTUNG"
    """optionale zusätzliche Zähleinrichtung"""
    MESSWANDLERSATZ_IMS_MME = "MESSWANDLERSATZ_IMS_MME"
    """Messwandlersatz Strom iMS und mME, NSP"""
    KOMBIMESSWANDLER_IMS_MME = "KOMBIMESSWANDLER_IMS_MME"
    """Kombimesswandlersatz (Strom u. Spg) iMS und mME"""
    TARIFSCHALTGERAET_IMS_MME = "TARIFSCHALTGERAET_IMS_MME"
    """Tarifschaltung iMS und mME"""
    RUNDSTEUEREMPFAENGER_IMS_MME = "RUNDSTEUEREMPFAENGER_IMS_MME"
    """Rundsteuerempfänger iMS und mME"""
    TEMPERATUR_KOMPENSATION = "TEMPERATUR_KOMPENSATION"
    """Temperaturkompensation"""
    HOECHSTBELASTUNGS_ANZEIGER = "HOECHSTBELASTUNGS_ANZEIGER"
    """Höchsbelastungsanzeiger"""
    SONSTIGES_GERAET = "SONSTIGES_GERAET"
    """Sonstiges Gerät"""
    EDL_21 = "EDL_21"
    """EDL21"""
    EDL_40_ZAEHLERAUFSATZ = "EDL_40_ZAEHLERAUFSATZ"
    """EDL 40 Zähleraufsatz für Zähler"""
    EDL_40 = "EDL_40"
    """EDL 40"""
    TELEFONANSCHLUSS = "TELEFONANSCHLUSS"
    """Telefonanschluss"""
    MODEM_GSM = "MODEM_GSM"
    """Modem-GSM"""
    MODEM_GPRS = "MODEM_GPRS"
    """Modem-GPRS"""
    MODEM_FUNK = "MODEM_FUNK"
    """Modem-Funk"""
    MODEM_GSM_O_LG = "MODEM_GSM_O_LG"
    """vom Messstellenbetreiber beigestelltes GSM-Modem ohne Lastgangmessung"""
    MODEM_GSM_M_LG = "MODEM_GSM_M_LG"
    """vom Messstellenbetreiber beigestelltes GSM-Modem mit Lastgangmessung"""
    MODEM_FESTNETZ = "MODEM_FESTNETZ"
    """vom Messstellenbetreiber beigestelltes Festnetz-Modem"""
    MODEM_GPRS_M_LG = "MODEM_GPRS_M_LG"
    """vom Messstellenbetreiber bereitgestelltes GPRS-Modem Lastgangmessung"""
    PLC_KOM = "PLC_KOM"
    """PLC-Kom.-Einrichtung (Powerline Communication)"""
    ETHERNET_KOM = "ETHERNET_KOM"
    """Ethernet-Kom.-Einricht. LAN/WLAN"""
    DSL_KOM = "DSL_KOM"
    """DSL-Kommunikationseinrichtung"""
    LTE_KOM = "LTE_KOM"
    """LTE-Kommunikationseinrichtung"""
    KOMPAKT_MU = "KOMPAKT_MU"
    """Kompaktmengenumwerter"""
    SYSTEM_MU = "SYSTEM_MU"
    """Systemmengenumwerter"""
    TEMPERATUR_MU = "TEMPERATUR_MU"
    """Temperaturmengenumwerter"""
    ZUSTANDS_MU = "ZUSTANDS_MU"
    """Zustandsmengenumwerter"""



"""
Contains Zeitraum class
"""

from datetime import date, time
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.mengeneinheit import Mengeneinheit


# pylint: disable=too-few-public-methods
@postprocess_docstring
class Zeitraum(COM):
    """
    Diese Komponente wird zur Abbildung von Zeiträumen in Form von Dauern oder der Angabe von Start und Ende verwendet.
    Es muss daher eine der drei Möglichkeiten angegeben sein:
    - Einheit und Dauer oder
    - Zeitraum: Startdatum bis Enddatum oder
    - Zeitraum: Startzeitpunkt (Datum und Uhrzeit) bis Endzeitpunkt (Datum und Uhrzeit)

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zeitraum.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zeitraum JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zeitraum.json>`_

    """

    typ: Annotated[Literal[ComTyp.ZEITRAUM], Field(alias="_typ")] = ComTyp.ZEITRAUM

    startdatum: Optional[date] = None
    """Startdatum des betrachteten Zeitraums ist **inklusiv**.

    Example:
        '2025-01-01'
    """
    enddatum: Optional[date] = None
    """Enddatum des betrachteten Zeitraums ist **inklusiv**.

    Example:
        '2025-01-01'
    """
    startuhrzeit: Optional[time] = None
    """Startuhrzeit mit Zeitzone. Die angegebene Uhrzeit ist im betrachteten Zeitraum **inklusiv**.

    Example:
        '18:00:00+01:00'
    """
    enduhrzeit: Optional[time] = None
    """Enduhrzeit mit Zeitzone. Die angegebene Uhrzeit ist im betrachteten Zeitraum **exklusiv**.

    Example:
        '19:00:00+01:00'
    """
    dauer: Optional[str] = None
    """
    Dauer in ISO 8601 Format.

    Example:
        'P1DT30H4S'

    See `RFC 3339 <https://datatracker.ietf.org/doc/html/rfc3339>`_
    """



"""
Contains Rechnungsposition class
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.bdewartikelnummer import BDEWArtikelnummer
    from ..enum.mengeneinheit import Mengeneinheit
    from .betrag import Betrag
    from .menge import Menge
    from .preis import Preis
    from .steuerbetrag import Steuerbetrag


@postprocess_docstring
class Rechnungsposition(COM):
    """
    Über Rechnungspositionen werden Rechnungen strukturiert.
    In einem Rechnungsteil wird jeweils eine in sich geschlossene Leistung abgerechnet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Rechnungsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Rechnungsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Rechnungsposition.json>`_

    """

    typ: Annotated[Literal[ComTyp.RECHNUNGSPOSITION], Field(alias="_typ")] = ComTyp.RECHNUNGSPOSITION

    positionsnummer: Optional[int] = None
    """Fortlaufende Nummer für die Rechnungsposition"""

    lieferung_von: Optional[pydantic.AwareDatetime] = None
    """Start der Lieferung für die abgerechnete Leistung (inklusiv)"""
    lieferung_bis: Optional[pydantic.AwareDatetime] = None
    """Ende der Lieferung für die abgerechnete Leistung (exklusiv)"""

    positionstext: Optional[str] = None
    """Bezeichung für die abgerechnete Position"""

    positions_menge: Optional["Menge"] = None
    """Die abgerechnete Menge mit Einheit"""
    einzelpreis: Optional["Preis"] = None
    """Der Preis für eine Einheit der energetischen Menge"""

    teilsumme_netto: Optional["Betrag"] = None
    """
    Das Ergebnis der Multiplikation aus einzelpreis * positionsMenge * (Faktor aus zeitbezogeneMenge).
    Z.B. 12,60€ * 120 kW * 3/12 (für 3 Monate).
    """
    # the cross check in general doesn't work because Betrag and Preis use different enums to describe the currency
    # see https://github.com/Hochfrequenz/BO4E-python/issues/126

    teilsumme_steuer: Optional["Steuerbetrag"] = None
    """Auf die Position entfallende Steuer, bestehend aus Steuersatz und Betrag"""

    zeiteinheit: Optional["Mengeneinheit"] = None
    """Falls sich der Preis auf eine Zeit bezieht, steht hier die Einheit"""

    artikelnummer: Optional["BDEWArtikelnummer"] = None
    """Kennzeichnung der Rechnungsposition mit der Standard-Artikelnummer des BDEW"""
    lokations_id: Optional[str] = None
    """Marktlokation, die zu dieser Position gehört"""

    zeitbezogene_menge: Optional["Menge"] = None
    """
    Eine auf die Zeiteinheit bezogene Untermenge.
    Z.B. bei einem Jahrespreis, 3 Monate oder 146 Tage.
    Basierend darauf wird der Preis aufgeteilt.
    """
    teilrabatt_netto: Optional["Betrag"] = None
    """Nettobetrag für den Rabatt dieser Position"""

    artikel_id: Optional[str] = None
    """Standardisierte vom BDEW herausgegebene Liste, welche im Strommarkt die BDEW-Artikelnummer ablöst"""



"""
Contains Zaehler class
"""

from decimal import Decimal

# pylint: disable=unused-argument
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.geraet import Geraet
    from ..com.zaehlwerk import Zaehlwerk
    from ..enum.befestigungsart import Befestigungsart
    from ..enum.messwerterfassung import Messwerterfassung
    from ..enum.registeranzahl import Registeranzahl
    from ..enum.sparte import Sparte
    from ..enum.zaehlerauspraegung import Zaehlerauspraegung
    from ..enum.zaehlergroesse import Zaehlergroesse
    from ..enum.zaehlertyp import Zaehlertyp
    from ..enum.zaehlertypspezifikation import ZaehlertypSpezifikation
    from .geschaeftspartner import Geschaeftspartner

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Zaehler(Geschaeftsobjekt):
    """
    Object containing information about a meter/"Zaehler".

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Zaehler.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehler JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Zaehler.json>`_

    """

    typ: Annotated[Literal[BoTyp.ZAEHLER], Field(alias="_typ")] = BoTyp.ZAEHLER
    zaehlernummer: Optional[str] = None
    """Nummerierung des Zählers,vergeben durch den Messstellenbetreiber"""
    sparte: Optional["Sparte"] = None
    """Strom oder Gas"""
    zaehlerauspraegung: Optional["Zaehlerauspraegung"] = None
    """Spezifikation die Richtung des Zählers betreffend"""
    zaehlertyp: Optional["Zaehlertyp"] = None
    """Typisierung des Zählers"""
    zaehlwerke: Optional[list["Zaehlwerk"]] = None
    registeranzahl: Optional["Registeranzahl"] = None
    """Spezifikation bezüglich unterstützter Tarif"""
    zaehlerkonstante: Optional[Decimal] = None
    """Zählerkonstante auf dem Zähler"""
    eichung_bis: Optional[pydantic.AwareDatetime] = None
    """Bis zu diesem Datum (exklusiv) ist der Zähler geeicht."""
    letzte_eichung: Optional[pydantic.AwareDatetime] = None
    """Zu diesem Datum fand die letzte Eichprüfung des Zählers statt."""
    zaehlerhersteller: Optional["Geschaeftspartner"] = None
    """Der Hersteller des Zählers"""
    ist_fernschaltbar: Optional[bool] = None
    """Fernschaltung"""
    messwerterfassung: Optional["Messwerterfassung"] = None
    """Messwerterfassung des Zählers"""
    zaehlertypSpezifikation: Optional["ZaehlertypSpezifikation"] = None
    """Besondere Spezifikation des Zählers"""
    befestigungsart: Optional["Befestigungsart"] = None
    """Befestigungsart"""
    zaehlergroesse: Optional["Zaehlergroesse"] = None
    """Größe des Zählers"""
    geraete: Optional[list["Geraet"]] = None
    """Liste der Geräte, die zu diesem Zähler gehören, bspw. Smartmeter-Gateway"""



"""
Contains Messlokation class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.geraet import Geraet
    from ..bo.lokationszuordnung import Lokationszuordnung
    from ..com.adresse import Adresse
    from ..com.dienstleistung import Dienstleistung
    from ..com.geokoordinaten import Geokoordinaten
    from ..com.katasteradresse import Katasteradresse
    from ..enum.netzebene import Netzebene
    from ..enum.sparte import Sparte
    from .zaehler import Zaehler

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Messlokation(Geschaeftsobjekt):
    """
    Object containing information about a Messlokation

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Messlokation.svg" type="image/svg+xml"></object>

    .. HINT::
        `Messlokation JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Messlokation.json>`_

    """

    typ: Annotated[Literal[BoTyp.MESSLOKATION], Field(alias="_typ")] = BoTyp.MESSLOKATION

    messlokations_id: Optional[str] = None
    """Die Messlokations-Identifikation; Das ist die frühere Zählpunktbezeichnung"""
    sparte: Optional["Sparte"] = None
    """Sparte der Messlokation, z.B. Gas oder Strom"""

    netzebene_messung: Optional["Netzebene"] = None
    """Spannungsebene der Messung"""
    messgebietnr: Optional[str] = None
    """Die Nummer des Messgebietes in der ene't-Datenbank"""
    geraete: Optional[list["Geraet"]] = None
    """Liste der Geräte, die zu dieser Messstelle gehört"""
    messdienstleistung: Optional[list["Dienstleistung"]] = None
    """Liste der Messdienstleistungen, die zu dieser Messstelle gehört"""  # todo: rename to plural
    messlokationszaehler: Optional[list["Zaehler"]] = None
    """Zähler, die zu dieser Messlokation gehören"""

    # only one of the following two optional codenr attributes can be set
    grundzustaendiger_msb_codenr: Optional[str] = None
    """
    Codenummer des grundzuständigen Messstellenbetreibers, der für diese Messlokation zuständig ist.
    (Dieser ist immer dann Messstellenbetreiber, wenn kein anderer MSB die Einrichtungen an der Messlokation betreibt.)
    """
    grundzustaendiger_msbim_codenr: Optional[str] = None
    """
    Codenummer des grundzuständigen Messstellenbetreibers für intelligente Messsysteme, der für diese Messlokation
    zuständig ist.
    (Dieser ist immer dann Messstellenbetreiber, wenn kein anderer MSB die Einrichtungen an der Messlokation betreibt.)
    """
    # only one of the following three optional address attributes can be set
    messadresse: Optional["Adresse"] = None
    """
    Die Adresse, an der die Messeinrichtungen zu finden sind.
    (Nur angeben, wenn diese von der Adresse der Marktlokation abweicht.)
    """
    geoadresse: Optional["Geokoordinaten"] = None
    """
    Alternativ zu einer postalischen Adresse kann hier ein Ort mittels Geokoordinaten angegeben werden
    (z.B. zur Identifikation von Sendemasten).
    """
    katasterinformation: Optional["Katasteradresse"] = None
    """
    Alternativ zu einer postalischen Adresse und Geokoordinaten kann hier eine Ortsangabe mittels Gemarkung und
    Flurstück erfolgen.
    """
    lokationszuordnungen: Optional[list["Lokationszuordnung"]] = None
    """Lokationszuordnung, um bspw. die zugehörigen Marktlokationen anzugeben"""
    lokationsbuendel_objektcode: Optional[str] = None
    """Lokationsbuendel Code, der die Funktion dieses BOs an der Lokationsbuendelstruktur beschreibt."""



# pylint: disable=missing-module-docstring,line-too-long
from bo4e.enum.strenum import StrEnum


class Rechnungsstatus(StrEnum):
    """
    Abbildung verschiedener Zustände, die im Rahmen der Rechnungsbearbeitung durchlaufen werden.
    """

    UNGEPRUEFT = "UNGEPRUEFT"
    """Die Rechnung wurde erstellt bzw ist eingegangen, wurde aber noch nicht geprüft."""
    GEPRUEFT_OK = "GEPRUEFT_OK"
    """Die Rechnung wurde geprüft und als korrekt befunden."""
    GEPRUEFT_FEHLERHAFT = "GEPRUEFT_FEHLERHAFT"
    """Bei der Prüfung der Rechnung hat sich herausgestellt, dass die Rechnung Fehler aufweist."""
    GEBUCHT = "GEBUCHT"
    """Die Buchhaltung hat die Rechnung aufgenommen und berücksichtigt."""
    BEZAHLT = "BEZAHLT"
    """Die Rechnung wurde beglichen."""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Verwendungszweck(StrEnum):
    """
    Verwendungszweck der Werte Marktlokation
    """

    NETZNUTZUNGSABRECHNUNG = "NETZNUTZUNGSABRECHNUNG"
    """Netznutzungsabrechnung"""
    BILANZKREISABRECHNUNG = "BILANZKREISABRECHNUNG"
    """Bilanzkreisabrechnung"""
    MEHRMINDERMENGENABRECHNUNG = "MEHRMINDERMENGENABRECHNUNG"
    """Mehrmindermengenabrechnung"""
    ENDKUNDENABRECHNUNG = "ENDKUNDENABRECHNUNG"
    """Endkundenabrechnung"""
    UEBERMITTLUNG_AN_DAS_HKNR = "UEBERMITTLUNG_AN_DAS_HKNR"
    """Übermittlung an das Herkunftsnachweisregister (HKNR)"""
    ERMITTLUNG_AUSGEGLICHENHEIT_BILANZKREIS = "ERMITTLUNG_AUSGEGLICHENHEIT_BILANZKREIS"
    """Zur Ermittlung der Ausgeglichenheit von Bilanzkreisen"""



"""
Contains Angebot class
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.angebotsvariante import Angebotsvariante
    from ..enum.sparte import Sparte
    from .geschaeftspartner import Geschaeftspartner
    from .person import Person


@postprocess_docstring
class Angebot(Geschaeftsobjekt):
    """
    Mit diesem BO kann ein Versorgungsangebot zur Strom- oder Gasversorgung oder die Teilnahme an einer Ausschreibung
    übertragen werden. Es können verschiedene Varianten enthalten sein (z.B. ein- und mehrjährige Laufzeit).
    Innerhalb jeder Variante können Teile enthalten sein, die jeweils für eine oder mehrere Marktlokationen erstellt
    werden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Angebot.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebot JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Angebot.json>`_


    """

    typ: Annotated[Literal[BoTyp.ANGEBOT], Field(alias="_typ")] = BoTyp.ANGEBOT
    angebotsnummer: Optional[str] = None
    """Eindeutige Nummer des Angebotes"""
    angebotsdatum: Optional[pydantic.AwareDatetime] = None
    """Erstellungsdatum des Angebots"""
    sparte: Optional["Sparte"] = None
    """Sparte, für die das Angebot abgegeben wird (Strom/Gas)"""
    angebotsgeber: Optional["Geschaeftspartner"] = None
    """Ersteller des Angebots"""
    angebotsnehmer: Optional["Geschaeftspartner"] = None
    """Empfänger des Angebots"""

    varianten: Optional[list["Angebotsvariante"]] = None
    """
    Eine oder mehrere Varianten des Angebots mit den Angebotsteilen;
    Ein Angebot besteht mindestens aus einer Variante.
    """

    anfragereferenz: Optional[str] = None
    """
    Referenz auf eine Anfrage oder Ausschreibung;
    Kann dem Empfänger des Angebotes bei Zuordnung des Angebotes zur Anfrage bzw. Ausschreibung helfen.
    """
    bindefrist: Optional[pydantic.AwareDatetime] = None
    """Bis zu diesem Zeitpunkt (Tag/Uhrzeit) inklusive gilt das Angebot"""
    unterzeichner_angebotsnehmer: Optional["Person"] = None
    """Person, die als Angebotsnehmer das Angebot angenommen hat"""
    unterzeichner_angebotsgeber: Optional["Person"] = None
    """Person, die als Angebotsgeber das Angebots ausgestellt hat"""



"""
Contains Preisposition class
"""

from decimal import Decimal

# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.bdewartikelnummer import BDEWArtikelnummer
    from ..enum.bemessungsgroesse import Bemessungsgroesse
    from ..enum.kalkulationsmethode import Kalkulationsmethode
    from ..enum.leistungstyp import Leistungstyp
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.tarifzeit import Tarifzeit
    from ..enum.waehrungseinheit import Waehrungseinheit
    from .preisstaffel import Preisstaffel

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Preisposition(COM):
    """
    Preis für eine definierte Lieferung oder Leistung innerhalb eines Preisblattes

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preisposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preisposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Preisposition.json>`_

    """

    typ: Annotated[Literal[ComTyp.PREISPOSITION], Field(alias="_typ")] = ComTyp.PREISPOSITION

    berechnungsmethode: Optional["Kalkulationsmethode"] = None
    """Das Modell, das der Preisbildung zugrunde liegt"""
    leistungstyp: Optional["Leistungstyp"] = None
    """Standardisierte Bezeichnung für die abgerechnete Leistungserbringung"""  #
    leistungsbezeichnung: Optional[str] = None
    """Bezeichnung für die in der Position abgebildete Leistungserbringung"""
    preiseinheit: Optional["Waehrungseinheit"] = None
    """Festlegung, mit welcher Preiseinheit abgerechnet wird, z.B. Ct. oder €"""
    bezugsgroesse: Optional["Mengeneinheit"] = None
    """Hier wird festgelegt, auf welche Bezugsgrösse sich der Preis bezieht, z.B. kWh oder Stück"""
    preisstaffeln: Optional[list["Preisstaffel"]] = None
    """Preisstaffeln, die zu dieser Preisposition gehören"""

    zeitbasis: Optional["Mengeneinheit"] = None
    """
    Die Zeit(dauer) auf die sich der Preis bezieht.
    Z.B. ein Jahr für einen Leistungspreis der in €/kW/Jahr ausgegeben wird
    """
    tarifzeit: Optional["Tarifzeit"] = None
    """Festlegung, für welche Tarifzeit der Preis hier festgelegt ist"""
    bdew_artikelnummer: Optional["BDEWArtikelnummer"] = None
    """
    Eine vom BDEW standardisierte Bezeichnug für die abgerechnete Leistungserbringung;
    Diese Artikelnummer wird auch im Rechnungsteil der INVOIC verwendet.
    """
    zonungsgroesse: Optional["Bemessungsgroesse"] = None
    """Mit der Menge der hier angegebenen Größe wird die Staffelung/Zonung durchgeführt. Z.B. Vollbenutzungsstunden"""
    freimenge_blindarbeit: Optional[Decimal] = None
    """Der Anteil der Menge der Blindarbeit in Prozent von der Wirkarbeit, für die keine Abrechnung erfolgt"""
    freimenge_leistungsfaktor: Optional[Decimal] = None
    """
    Der cos phi (Verhältnis Wirkleistung/Scheinleistung) aus dem die Freimenge für die Blindarbeit berechnet wird als
    tan phi (Verhältnis Blindleistung/Wirkleistung)
    """
    gruppenartikel_id: Optional[str] = None
    """
    Übergeordnete Gruppen-ID, die sich ggf. auf die Artikel-ID in der Preisstaffel bezieht
    """



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Waehrungscode(StrEnum):
    """
    Aufzählung der Währungscodes.
    """

    AED = "AED"
    """UAE Dirham"""
    AFN = "AFN"
    """Afghani"""
    ALL = "ALL"
    """Lek"""
    AMD = "AMD"
    """Armenian Dram"""
    ANG = "ANG"
    """Netherlands Antillean Guilder"""
    AOA = "AOA"
    """Kwanza"""
    ARS = "ARS"
    """Argentine Peso"""
    AUD = "AUD"
    """Australian Dollar"""
    AWG = "AWG"
    """Aruban Florin"""
    AZN = "AZN"
    """Azerbaijanian Manat"""
    BAM = "BAM"
    """Convertible Mark"""
    BBD = "BBD"
    """Barbados Dollar"""
    BDT = "BDT"
    """Taka"""
    BGN = "BGN"
    """Bulgarian Lev"""
    BHD = "BHD"
    """Bahraini Dinar"""
    BIF = "BIF"
    """Burundi Franc"""
    BMD = "BMD"
    """Bermudian Dollar"""
    BND = "BND"
    """Brunei Dollar"""
    BOB = "BOB"
    """Boliviano"""
    BOV = "BOV"
    """Mvdol"""
    BRL = "BRL"
    """Brazilian Real"""
    BSD = "BSD"
    """Bahamian Dollar"""
    BTN = "BTN"
    """Ngultrum"""
    BWP = "BWP"
    """Pula"""
    BYN = "BYN"
    """Belarusian Ruble"""
    BYR = "BYR"
    """Belarusian Ruble"""
    BZD = "BZD"
    """Belize Dollar"""
    CAD = "CAD"
    """Canadian Dollar"""
    CDF = "CDF"
    """Congolese Franc"""
    CHE = "CHE"
    """WIR Euro"""
    CHF = "CHF"
    """Swiss Franc"""
    CHW = "CHW"
    """WIR Franc"""
    CLF = "CLF"
    """Unidad de Fomento"""
    CLP = "CLP"
    """Chilean Peso"""
    CNY = "CNY"
    """Yuan Renminbi"""
    COP = "COP"
    """Colombian Peso"""
    COU = "COU"
    """Unidad de Valor Real"""
    CRC = "CRC"
    """Costa Rican Colon"""
    CUC = "CUC"
    """Peso Convertible"""
    CUP = "CUP"
    """Cuban Peso"""
    CVE = "CVE"
    """Cape Verde Escudo"""
    CZK = "CZK"
    """Czech Koruna"""
    DJF = "DJF"
    """Djibouti Franc"""
    DKK = "DKK"
    """Danish Krone"""
    DOP = "DOP"
    """Dominican Peso"""
    DZD = "DZD"
    """Algerian Dinar"""
    EGP = "EGP"
    """Egyptian Pound"""
    ERN = "ERN"
    """Nakfa"""
    ETB = "ETB"
    """Ethiopian Birr"""
    EUR = "EUR"
    """Euro"""
    FJD = "FJD"
    """Fiji Dollar"""
    FKP = "FKP"
    """Falkland Islands Pound"""
    GBP = "GBP"
    """Pound Sterling"""
    GEL = "GEL"
    """Lari"""
    GHS = "GHS"
    """Ghana Cedi"""
    GIP = "GIP"
    """Gibraltar Pound"""
    GMD = "GMD"
    """Dalasi"""
    GNF = "GNF"
    """Guinea Franc"""
    GTQ = "GTQ"
    """Quetzal"""
    GYD = "GYD"
    """Guyana Dollar"""
    HKD = "HKD"
    """Hong Kong Dollar"""
    HNL = "HNL"
    """Lempira"""
    HRK = "HRK"
    """Croatian Kuna"""
    HTG = "HTG"
    """Gourde"""
    HUF = "HUF"
    """Forint"""
    IDR = "IDR"
    """Rupiah"""
    ILS = "ILS"
    """New Israeli Sheqel"""
    INR = "INR"
    """Indian Rupee"""
    IQD = "IQD"
    """Iraqi Dinar"""
    IRR = "IRR"
    """Iranian Rial"""
    ISK = "ISK"
    """Iceland Krona"""
    JMD = "JMD"
    """Jamaican Dollar"""
    JOD = "JOD"
    """Jordanian Dinar"""
    JPY = "JPY"
    """Yen"""
    KES = "KES"
    """Kenyan Shilling"""
    KGS = "KGS"
    """Som"""
    KHR = "KHR"
    """Riel"""
    KMF = "KMF"
    """Comoro Franc"""
    KPW = "KPW"
    """North Korean Won"""
    KRW = "KRW"
    """Won"""
    KWD = "KWD"
    """Kuwaiti Dinar"""
    KYD = "KYD"
    """Cayman Islands Dollar"""
    KZT = "KZT"
    """Tenge"""
    LAK = "LAK"
    """Kip"""
    LBP = "LBP"
    """Lebanese Pound"""
    LKR = "LKR"
    """Sri Lanka Rupee"""
    LRD = "LRD"
    """Liberian Dollar"""
    LSL = "LSL"
    """Loti"""
    LTL = "LTL"
    """Lithuanian Litas"""
    LYD = "LYD"
    """Libyan Dinar"""
    MAD = "MAD"
    """Moroccan Dirham"""
    MDL = "MDL"
    """Moldovan Leu"""
    MGA = "MGA"
    """Malagasy Ariary"""
    MKD = "MKD"
    """Denar"""
    MMK = "MMK"
    """Kyat"""
    MNT = "MNT"
    """Tugrik"""
    MOP = "MOP"
    """Pataca"""
    MRO = "MRO"
    """Ouguiya"""
    MUR = "MUR"
    """Mauritius Rupee"""
    MVR = "MVR"
    """Rufiyaa"""
    MWK = "MWK"
    """Kwacha"""
    MXN = "MXN"
    """Mexican Peso"""
    MXV = "MXV"
    """Mexican Unidad de Inversion (UDI)"""
    MYR = "MYR"
    """Malaysian Ringgit"""
    MZN = "MZN"
    """Mozambique Metical"""
    NAD = "NAD"
    """Namibia Dollar"""
    NGN = "NGN"
    """Naira"""
    NIO = "NIO"
    """Cordoba Oro"""
    NOK = "NOK"
    """Norwegian Krone"""
    NPR = "NPR"
    """Nepalese Rupee"""
    NZD = "NZD"
    """New Zealand Dollar"""
    OMR = "OMR"
    """Rial Omani"""
    PAB = "PAB"
    """Balboa"""
    PEN = "PEN"
    """Nuevo Sol"""
    PGK = "PGK"
    """Kina"""
    PHP = "PHP"
    """Philippine Peso"""
    PKR = "PKR"
    """Pakistan Rupee"""
    PLN = "PLN"
    """Zloty"""
    PYG = "PYG"
    """Guarani"""
    QAR = "QAR"
    """Qatari Rial"""
    RON = "RON"
    """New Romanian Leu"""
    RSD = "RSD"
    """Serbian Dinar"""
    RUB = "RUB"
    """Russian Ruble"""
    RUR = "RUR"
    """Russian Ruble"""
    RWF = "RWF"
    """Rwanda Franc"""
    SAR = "SAR"
    """Saudi Riyal"""
    SBD = "SBD"
    """Solomon Islands Dollar"""
    SCR = "SCR"
    """Seychelles Rupee"""
    SDG = "SDG"
    """Sudanese Pound"""
    SEK = "SEK"
    """Swedish Krona"""
    SGD = "SGD"
    """Singapore Dollar"""
    SHP = "SHP"
    """Saint Helena Pound"""
    SLL = "SLL"
    """Leone"""
    SOS = "SOS"
    """Somali Shilling"""
    SRD = "SRD"
    """Surinam Dollar"""
    SSP = "SSP"
    """South Sudanese Pound"""
    STD = "STD"
    """Dobra"""
    SVC = "SVC"
    """El Salvador Colon"""
    SYP = "SYP"
    """Syrian Pound"""
    SZL = "SZL"
    """Lilangeni"""
    THB = "THB"
    """Baht"""
    TJS = "TJS"
    """Somoni"""
    TMT = "TMT"
    """Turkmenistan New Manat"""
    TND = "TND"
    """Tunisian Dinar"""
    TOP = "TOP"
    """Paʻanga"""
    TRY = "TRY"
    """Turkish Lira"""
    TTD = "TTD"
    """Trinidad and Tobago Dollar"""
    TWD = "TWD"
    """New Taiwan Dollar"""
    TZS = "TZS"
    """Tanzanian Shilling"""
    UAH = "UAH"
    """Hryvnia"""
    UGX = "UGX"
    """Uganda Shilling"""
    USD = "USD"
    """US Dollar"""
    USN = "USN"
    """US Dollar (Next day)"""
    USS = "USS"
    """US Dollar (Same day)"""
    UYI = "UYI"
    """Uruguay Peso en Unidades Indexadas (URUIURUI)"""
    UYU = "UYU"
    """Peso Uruguayo"""
    UZS = "UZS"
    """Uzbekistan Sum"""
    VEF = "VEF"
    """Bolivar"""
    VND = "VND"
    """Dong"""
    VUV = "VUV"
    """Vatu"""
    WST = "WST"
    """Tala"""
    XAF = "XAF"
    """CFA Franc BEAC"""
    XAG = "XAG"
    """Silver"""
    XAU = "XAU"
    """Gold"""
    XBA = "XBA"
    """Bond Markets Unit European Composite Unit (EURCO)"""
    XBB = "XBB"
    """Bond Markets Unit European Monetary Unit (E.M.U.-6)"""
    XBC = "XBC"
    """Bond Markets Unit European Unit of Account 9 (E.U.A.-9)"""
    XBD = "XBD"
    """Bond Markets Unit European Unit of Account 17 (E.U.A.-17)"""
    XCD = "XCD"
    """East Caribbean Dollar"""
    XDR = "XDR"
    """SDR (Special Drawing Right)"""
    XOF = "XOF"
    """CFA Franc BCEAO"""
    XPD = "XPD"
    """Palladium"""
    XPF = "XPF"
    """CFP Franc"""
    XPT = "XPT"
    """Platinum"""
    XSU = "XSU"
    """Sucre"""
    XTS = "XTS"
    """Codes specifically reserved for testing purposes"""
    XUA = "XUA"
    """ADB Unit of Account"""
    XXX = "XXX"
    """The codes assigned for transactions where no currency ist involved"""
    YER = "YER"
    """Yemeni Rial"""
    ZAR = "ZAR"
    """Rand"""
    ZMW = "ZMW"
    """Zambian Kwacha"""
    ZWL = "ZWL"
    """Zimbabwe Dollar"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Oekolabel(StrEnum):
    """
    Aufzählung der Labels für Öko-Strom von verschiedenen Herausgebern.
    """

    ENERGREEN = "ENERGREEN"
    """ENERGREEN"""
    GASGREEN_GRUENER_STROM = "GASGREEN_GRUENER_STROM"
    """GASGREEN_GRUENER_STROM"""
    GASGREEN = "GASGREEN"
    """GASGREEN"""
    GRUENER_STROM_GOLD = "GRUENER_STROM_GOLD"
    """GRUENER_STROM_GOLD"""
    GRUENER_STROM_SILBER = "GRUENER_STROM_SILBER"
    """GRUENER_STROM_SILBER"""
    GRUENER_STROM = "GRUENER_STROM"
    """GRUENER_STROM"""
    GRUENES_GAS = "GRUENES_GAS"
    """GRUENES_GAS"""
    NATURWATT_STROM = "NATURWATT_STROM"
    """NATURWATT_STROM"""
    OK_POWER = "OK_POWER"
    """OK_POWER"""
    RENEWABLE_PLUS = "RENEWABLE_PLUS"
    """RENEWABLE_PLUS"""
    WATERGREEN = "WATERGREEN"
    """WATERGREEN"""
    WATERGREEN_PLUS = "WATERGREEN_PLUS"
    """WATERGREEN_PLUS"""



from datetime import datetime, timezone

import pytest

from bo4e import Unterschrift
from tests.serialization_helper import assert_serialization_roundtrip


class TestUnterschrift:
    @pytest.mark.parametrize(
        "unterschrift",
        [
            pytest.param(
                Unterschrift(
                    name="Foo",
                    ort="Grünwald",
                    datum=datetime(2019, 6, 7, tzinfo=timezone.utc),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, unterschrift: Unterschrift) -> None:
        """
        Test de-/serialisation of Unterschrift.
        """
        assert_serialization_roundtrip(unterschrift)



"""
Contains Zaehlwerk class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..com.konzessionsabgabe import Konzessionsabgabe
    from ..com.verwendungszweckpromarktrolle import VerwendungszweckProMarktrolle
    from ..com.zaehlzeitregister import Zaehlzeitregister
    from ..enum.energierichtung import Energierichtung
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.verbrauchsart import Verbrauchsart
    from ..enum.waermenutzung import Waermenutzung


@postprocess_docstring
class Zaehlwerk(COM):
    """
    Mit dieser Komponente werden Zählwerke modelliert.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Zaehlwerk.svg" type="image/svg+xml"></object>

    .. HINT::
        `Zaehlwerk JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Zaehlwerk.json>`_

    """

    typ: Annotated[Literal[ComTyp.ZAEHLWERK], Field(alias="_typ")] = ComTyp.ZAEHLWERK

    zaehlwerk_id: Optional[str] = None
    """
    Identifikation des Zählwerks (Registers) innerhalb des Zählers.
    Oftmals eine laufende Nummer hinter der Zählernummer. Z.B. 47110815_1
    """
    bezeichnung: Optional[str] = None
    """Zusätzliche Bezeichnung, z.B. Zählwerk_Wirkarbeit."""
    richtung: Optional["Energierichtung"] = None
    """Die Energierichtung, Einspeisung oder Ausspeisung."""
    obis_kennzahl: Optional[str] = None
    """
    Die OBIS-Kennzahl für das Zählwerk, die festlegt, welche auf die gemessene Größe mit dem Stand gemeldet wird.
    Nur Zählwerkstände mit dieser OBIS-Kennzahl werden an diesem Zählwerk registriert.
    """
    wandlerfaktor: Optional[Decimal] = None
    """
    Mit diesem Faktor wird eine Zählerstandsdifferenz multipliziert, um zum eigentlichen Verbrauch im Zeitraum
    zu kommen.
    """
    einheit: Optional["Mengeneinheit"] = None
    """Die Einheit der gemessenen Größe, z.B. kWh"""
    ist_schwachlastfaehig: Optional[bool] = None
    """Schwachlastfaehigkeit"""
    verwendungszwecke: Optional[list["VerwendungszweckProMarktrolle"]] = None
    """Verwendungungszweck der Werte Marktlokation"""
    verbrauchsart: Optional["Verbrauchsart"] = None
    """Stromverbrauchsart/Verbrauchsart Marktlokation"""
    ist_unterbrechbar: Optional[bool] = None
    """Unterbrechbarkeit Marktlokation"""
    waermenutzung: Optional["Waermenutzung"] = None
    """Wärmenutzung Marktlokation"""
    konzessionsabgabe: Optional["Konzessionsabgabe"] = None
    """Konzessionsabgabe"""
    ist_steuerbefreit: Optional[bool] = None
    """Steuerbefreiung"""
    vorkommastelle: Optional[int] = None
    """Anzahl der Vorkommastellen"""
    nachkommastelle: Optional[int] = None
    """Anzahl der Nachkommastellen"""
    ist_abrechnungsrelevant: Optional[bool] = None
    """Abrechnungsrelevant"""
    anzahlAblesungen: Optional[int] = None
    """Anzahl Ablesungen pro Jahr"""
    zaehlzeitregister: Optional["Zaehlzeitregister"] = None
    """Erweiterte Definition der Zählzeit in Bezug auf ein Register"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class AbgabeArt(StrEnum):
    """
    Art der Konzessionsabgabe
    """

    KAS = "KAS"
    """KAS: für alle konzessionsvertraglichen Sonderregelungen, die nicht in die Systematik der KAV eingegliedert sind"""
    SA = "SA"
    """SA: Sondervertragskunden  1 kV, Preis nach § 2 (3) (für Strom 0,11 ct/kWh und für Gas 0,03 ct/kWh)"""
    SAS = "SAS"
    """SAS: Kennzeichnung, dass ein abweichender Preis für Sondervertragskunden vorliegt"""
    TA = "TA"
    """TA: Tarifkunden, für Strom § 2. (2) 1b HT bzw.ET(hohe KA) und für Gas § 2 (2) 2b"""
    TAS = "TAS"
    """TAS: Kennzeichnung, dass ein abweichender Preis für Tarifkunden vorliegt"""
    TK = "TK"
    """TK: für Gas nach KAV § 2 (2) 2a bei ausschließlicher Nutzung zum Kochen und Warmwassererzeugung"""
    TKS = "TKS"
    """TKS: Kennzeichnung, wenn nach KAV § 2 (2) 2a ein anderen Preis zu verwenden ist"""
    TS = "TS"
    """TS: für Strom mit Schwachlast § 2. (2) 1a NT(niedrige KA, 0,61 ct/kWh)"""
    TSS = "TSS"
    """TSS: Kennzeichnung, dass ein abweichender Preis für Schwachlast angewendet wird"""



"""
Version information for the bo4e package.
"""

import re
from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("bo4e")
    # Please keep this name in sync with the name of the project in pyproject.toml
    # This name is the name of the package on pypi.org
except PackageNotFoundError:
    __version__ = "0.0.0"


def _parse_version_to_gh_version(version_str: str) -> str:
    """
    Parse a version string into a GitHub version string.
    E.g. '202401.0.1-rc8+dev12asdf34' becomes 'v202401.0.1-rc8'.
    """
    _regex_version = re.compile(
        r"^(?P<major>\d{6})\."
        r"(?P<functional>\d+)\."
        r"(?P<technical>\d+)"
        r"(?:rc(?P<candidate>\d*))?"
        r"(?:\+g(?P<commit_part>\w+)"
        r"(?:\.d(?P<dirty_workdir_date_year>\d{4})"
        r"(?P<dirty_workdir_date_month>\d{2})"
        r"(?P<dirty_workdir_date_day>\d{2}))?)?$"
    )
    match = _regex_version.match(version_str)
    if match is None:
        raise ValueError(f"Invalid version string: {version_str}")

    return (
        f"v{match.group('major')}.{match.group('functional')}.{match.group('technical')}"
        + (f"-rc{match.group('candidate')}" if match.group("candidate") else "")
        + (f"+g{match.group('commit_part')}" if match.group("commit_part") else "")
        + (
            f".d{match.group('dirty_workdir_date_year')}"
            f"{match.group('dirty_workdir_date_month')}"
            f"{match.group('dirty_workdir_date_day')}"
            if match.group("dirty_workdir_date_year")
            else ""
        )
    )


__gh_version__ = _parse_version_to_gh_version(__version__)



"""
Contains Enums for Profilart.
"""

from bo4e.enum.strenum import StrEnum


class Profilart(StrEnum):
    """
    Profilart: temperaturabh./Standardlastprofil.
    """

    ART_STANDARDLASTPROFIL = "ART_STANDARDLASTPROFIL"  #: ART_STANDARDLASTPROFIL, Z02
    ART_TAGESPARAMETERABHAENGIGES_LASTPROFIL = "ART_TAGESPARAMETERABHAENGIGES_LASTPROFIL"
    """ART_TAGESPARAMETERABHAENGIGES_LASTPROFIL, Z03"""
    ART_LASTPROFIL = "ART_LASTPROFIL"  #: ART_LASTPROFIL, Z12
    ART_STANDARDEINSPEISEPROFIL = "ART_STANDARDEINSPEISEPROFIL"  #: ART_STANDARDEINSPEISEPROFIL, Z04
    ART_TAGESPARAMETERABHAENGIGES_EINSPEISEPROFIL = "ART_TAGESPARAMETERABHAENGIGES_EINSPEISEPROFIL"
    #: ART_TAGESPARAMETERABHAENGIGES_EINSPEISEPROFIL, Z05



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Preistyp(StrEnum):
    """
    Aufschlüsselung der Preistypen in Tarifen.
    """

    GRUNDPREIS = "GRUNDPREIS"
    """Grundpreis"""
    ARBEITSPREIS_EINTARIF = "ARBEITSPREIS_EINTARIF"
    """Arbeitspreis_ET"""
    ARBEITSPREIS_HT = "ARBEITSPREIS_HT"
    """Arbeitspreis_HT"""
    ARBEITSPREIS_NT = "ARBEITSPREIS_NT"
    """Arbeitspreis_NT"""
    LEISTUNGSPREIS = "LEISTUNGSPREIS"
    """Leistungspreis"""
    MESSPREIS = "MESSPREIS"
    """Messpreis"""
    ENTGELT_ABLESUNG = "ENTGELT_ABLESUNG"
    """Entgelt fuer Ablesung"""
    ENTGELT_ABRECHNUNG = "ENTGELT_ABRECHNUNG"
    """Entgelt fuer Abrechnung"""
    ENTGELT_MSB = "ENTGELT_MSB"
    """Entgelt für MSB (Entgelt für Einbau, Betrieb und Wartung der Messtechnik)"""
    PROVISION = "PROVISION"
    """Provision"""



import pytest

from bo4e import (
    Lokationszuordnung,
    Marktlokation,
    Messlokation,
    Netzlokation,
    SteuerbareRessource,
    TechnischeRessource,
    Zeitraum,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestLokationszuordnung:
    @pytest.mark.parametrize(
        "lokationszuordnung",
        [
            pytest.param(
                Lokationszuordnung(
                    marktlokationen=[Marktlokation()],
                    messlokationen=[Messlokation()],
                    netzlokationen=[Netzlokation()],
                    technische_ressourcen=[TechnischeRessource()],
                    steuerbare_ressourcen=[SteuerbareRessource()],
                    gueltigkeit=Zeitraum(),
                    zuordnungstyp="Zuordnungstyp",
                    lokationsbuendelcode="9992 00000 125 6",
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, lokationszuordnung: Lokationszuordnung) -> None:
        """
        Test de-/serialisation of Lokationszuordnung.
        """
        assert_serialization_roundtrip(lokationszuordnung)



from datetime import date, time, timedelta, timezone

import pytest

from bo4e import Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestZeitraum:
    @pytest.mark.parametrize(
        "zeitraum",
        [
            pytest.param(
                Zeitraum(
                    dauer="P1DT30H4S",
                ),
                id="period-P1DT30H4S",
            ),
            pytest.param(
                Zeitraum(
                    startdatum=date(2025, 1, 1),
                    enddatum=date(2025, 1, 31),
                ),
                id="startdatum-enddatum",
            ),
            pytest.param(
                Zeitraum(
                    startuhrzeit=time(10, 0, 0),
                    enduhrzeit=time(11, 0, 0),
                ),
                id="startuhrzeit-enduhrzeit",
            ),
            pytest.param(
                Zeitraum(
                    startuhrzeit=time(10, 0, 0, tzinfo=timezone.utc),
                    enduhrzeit=time(11, 0, 0, tzinfo=timezone.utc),
                ),
                id="startuhrzeit-enduhrzeit-utc-timezone",
            ),
            pytest.param(
                Zeitraum(
                    startuhrzeit=time(18, 0, 0, tzinfo=timezone(timedelta(hours=1), "Europe/Berlin")),
                    enduhrzeit=time(19, 0, 0, tzinfo=timezone(timedelta(hours=1), "Europe/Berlin")),
                ),
                id="startuhrzeit-enduhrzeit-berlin-timezone",
            ),
        ],
    )
    def test_serialization_roundtrip(self, zeitraum: Zeitraum) -> None:
        """
        Test de-/serialisation of Zeitraum.
        """
        assert_serialization_roundtrip(zeitraum)



"""
Contains Region class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.regionskriterium import Regionskriterium


# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Region(Geschaeftsobjekt):
    """
    Modellierung einer Region als Menge von Kriterien, die eine Region beschreiben

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Region.svg" type="image/svg+xml"></object>

    .. HINT::
        `Region JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Region.json>`_

    """

    typ: Annotated[Literal[BoTyp.REGION], Field(alias="_typ")] = BoTyp.REGION
    bezeichnung: Optional[str] = None
    """Bezeichnung der Region"""

    positiv_liste: Optional[list["Regionskriterium"]] = None
    """Positivliste der Kriterien zur Definition der Region"""

    negativ_liste: Optional[list["Regionskriterium"]] = None
    """Negativliste der Kriterien zur Definition der Region"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class BDEWArtikelnummer(StrEnum):
    """
    BDEW Artikelnummern
    """

    LEISTUNG = "LEISTUNG"
    """Leistung"""
    LEISTUNG_PAUSCHAL = "LEISTUNG_PAUSCHAL"
    """Leistung pauschal"""
    GRUNDPREIS = "GRUNDPREIS"
    """Grundpreis"""
    REGELENERGIE_ARBEIT = "REGELENERGIE_ARBEIT"
    """Regelenergie Arbeit"""
    REGELENERGIE_LEISTUNG = "REGELENERGIE_LEISTUNG"
    """Regelenergie Leistung"""
    NOTSTROMLIEFERUNG_ARBEIT = "NOTSTROMLIEFERUNG_ARBEIT"
    """Notstromlieferung Arbeit"""
    NOTSTROMLIEFERUNG_LEISTUNG = "NOTSTROMLIEFERUNG_LEISTUNG"
    """Notstromlieferung Leistung"""
    RESERVENETZKAPAZITAET = "RESERVENETZKAPAZITAET"
    """Reservenetzkapazität"""
    RESERVELEISTUNG = "RESERVELEISTUNG"
    """Reserveleistung"""
    ZUSAETZLICHE_ABLESUNG = "ZUSAETZLICHE_ABLESUNG"
    """Zusätzliche Ablesung"""
    PRUEFGEBUEHREN_AUSSERPLANMAESSIG = "PRUEFGEBUEHREN_AUSSERPLANMAESSIG"
    """Prüfgebühren (außerplanmäßig)"""
    WIRKARBEIT = "WIRKARBEIT"
    """Wirkarbeit"""
    SINGULAER_GENUTZTE_BETRIEBSMITTEL = "SINGULAER_GENUTZTE_BETRIEBSMITTEL"
    """singulär genutzte Betriebsmittel (z. B. Trafomiete, Leitungen)"""
    ABGABE_KWKG = "ABGABE_KWKG"
    """Abgabe KWKG"""
    ABSCHLAG = "ABSCHLAG"
    """Abschlag"""
    KONZESSIONSABGABE = "KONZESSIONSABGABE"
    """Konzessionsabgabe"""
    ENTGELT_FERNAUSLESUNG = "ENTGELT_FERNAUSLESUNG"
    """Entgelt für Fernauslesung"""
    UNTERMESSUNG = "UNTERMESSUNG"
    """Untermessung"""
    BLINDMEHRARBEIT = "BLINDMEHRARBEIT"
    """Blindmehrarbeit"""
    ENTGELT_ABRECHNUNG = "ENTGELT_ABRECHNUNG"
    """Entgelt für Abrechnung"""
    SPERRKOSTEN = "SPERRKOSTEN"
    """Sperrkosten"""
    ENTSPERRKOSTEN = "ENTSPERRKOSTEN"
    """Entsperrkosten"""
    MAHNKOSTEN = "MAHNKOSTEN"
    """Mahnkosten"""
    MEHR_MINDERMENGEN = "MEHR_MINDERMENGEN"
    """Mehr- und Mindermenge"""
    INKASSOKOSTEN = "INKASSOKOSTEN"
    """Inkassokosten"""
    BLINDMEHRLEISTUNG = "BLINDMEHRLEISTUNG"
    """Blindmehrleistung"""
    ENTGELT_MESSUNG_ABLESUNG = "ENTGELT_MESSUNG_ABLESUNG"
    """Entgelt für Messung und Ablesung"""
    ENTGELT_EINBAU_BETRIEB_WARTUNG_MESSTECHNIK = "ENTGELT_EINBAU_BETRIEB_WARTUNG_MESSTECHNIK"
    """Entgelt für Einbau, Betrieb und Wartung der Messtechnik"""
    AUSGLEICHSENERGIE = "AUSGLEICHSENERGIE"
    """Ausgleichsenergie"""
    ZAEHLEINRICHTUNG = "ZAEHLEINRICHTUNG"
    """Zähleinrichtung"""
    WANDLER_MENGENUMWERTER = "WANDLER_MENGENUMWERTER"
    """Wandler/Mengenumwerter"""
    KOMMUNIKATIONSEINRICHTUNG = "KOMMUNIKATIONSEINRICHTUNG"
    """Kommunikationseinrichtung"""
    TECHNISCHE_STEUEREINRICHTUNG = "TECHNISCHE_STEUEREINRICHTUNG"
    """Technische Steuereinrichtung"""
    PARAGRAF_19_STROM_NEV_UMLAGE = "PARAGRAF_19_STROM_NEV_UMLAGE"
    """§ 19 StromNEV Umlage"""
    BEFESTIGUNGSEINRICHTUNG = "BEFESTIGUNGSEINRICHTUNG"
    """Befestigungseinrichtung (z. B. Zählertafel)"""
    OFFSHORE_HAFTUNGSUMLAGE = "OFFSHORE_HAFTUNGSUMLAGE"
    """Offshore-Haftungsumlage"""
    FIXE_ARBEITSENTGELTKOMPONENTE = "FIXE_ARBEITSENTGELTKOMPONENTE"
    """Fixe Arbeitsentgeltkomponente"""
    FIXE_LEISTUNGSENTGELTKOMPONENTE = "FIXE_LEISTUNGSENTGELTKOMPONENTE"
    """Fixe Leistungsentgeltkomponente"""
    UMLAGE_ABSCHALTBARE_LASTEN = "UMLAGE_ABSCHALTBARE_LASTEN"
    """Umlage abschaltbare Lasten"""
    MEHRMENGE = "MEHRMENGE"
    """Mehrmenge"""
    MINDERMENGE = "MINDERMENGE"
    """Mindermenge"""
    ENERGIESTEUER = "ENERGIESTEUER"
    """Energiesteuer"""
    SMARTMETER_GATEWAY = "SMARTMETER_GATEWAY"
    """Smartmeter-Gateway"""
    STEUERBOX = "STEUERBOX"
    """Steuerbox"""
    MSB_INKL_MESSUNG = "MSB_INKL_MESSUNG"
    """Messtellenbetrieb inklusive Messung"""
    AUSGLEICHSENERGIE_UNTERDECKUNG = "AUSGLEICHSENERGIE_UNTERDECKUNG"
    """AusgleichsenergieUnterdeckung"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Bemessungsgroesse(StrEnum):
    """
    Zur Abbildung von Messgrössen und zur Verwendung in energiewirtschaftlichen Berechnungen.
    """

    WIRKARBEIT_EL = "WIRKARBEIT_EL"
    """elektrische Wirkarbeit"""
    LEISTUNG_EL = "LEISTUNG_EL"
    """elektrische Leistung"""
    BLINDARBEIT_KAP = "BLINDARBEIT_KAP"
    """Blindarbeit kapazitiv"""
    BLINDARBEIT_IND = "BLINDARBEIT_IND"
    """Blindarbeit induktiv"""
    BLINDLEISTUNG_KAP = "BLINDLEISTUNG_KAP"
    """Blindleistung kapazitiv"""
    BLINDLEISTUNG_IND = "BLINDLEISTUNG_IND"
    """Blindleistung induktiv"""
    WIRKARBEIT_TH = "WIRKARBEIT_TH"
    """thermische Wirkarbeit"""
    LEISTUNG_TH = "LEISTUNG_TH"
    """thermische Leistung"""
    VOLUMEN = "VOLUMEN"
    """Volumen"""
    VOLUMENSTROM = "VOLUMENSTROM"
    """Volumenstrom (Volumen/Zeit)"""
    BENUTZUNGSDAUER = "BENUTZUNGSDAUER"
    """Benutzungsdauer (Arbeit/Leistung)"""
    ANZAHL = "ANZAHL"
    """Darstellung einer Stückzahl"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Organisationstyp(StrEnum):
    """
    Hier wird festgelegt, ob der Geschäftspartner eine Person, eine Firma oder etwas anderes ist.
    """

    PRIVATPERSON = "PRIVATPERSON"
    """B2C"""
    UNTERNEHMEN = "UNTERNEHMEN"
    """B2B"""
    KOMMUNALE_EINRICHTUNG = "KOMMUNALE_EINRICHTUNG"
    """B2A"""
    STAATLICHE_BEHOERDE = "STAATLICHE_BEHOERDE"
    """B2G"""



import pytest

from bo4e import StandorteigenschaftenStrom
from tests.serialization_helper import assert_serialization_roundtrip


class TestStandorteigenschaftenStrom:
    @pytest.mark.parametrize(
        "standorteigenschaften_strom",
        [
            pytest.param(
                StandorteigenschaftenStrom(
                    regelzone="Transnet BW",
                    bilanzierungsgebiet_eic="11YW-ALBSTADT--5",
                    regelzone_eic="10YDE-ENBW-----N",
                ),
            )
        ],
    )
    def test_serialization_roundtrip(self, standorteigenschaften_strom: StandorteigenschaftenStrom) -> None:
        """
        Test de-/serialisation.
        """
        assert_serialization_roundtrip(standorteigenschaften_strom)
